/*
 * UUID: fb361a64-ec52-11dc-846c-00502c05c241
 * Author: Brian M. Ames, bames@apk.net
 * Copyright: Copyright (C) 2008 by Brian M. Ames
 */

#ifndef _fb361a64_ec52_11dc_846c_00502c05c241_
#define _fb361a64_ec52_11dc_846c_00502c05c241_

void goto_incl_first();
char *get_incl_path();
int goto_incl_next();
void incl_path_used();
void add_incl_path(char *p);

#endif //_fb361a64_ec52_11dc_846c_00502c05c241_
