#ifndef _bb188d3c_e320_11dc_8b9d_00502c05c241_
#define _bb188d3c_e320_11dc_8b9d_00502c05c241_

#include "tree.h"

void dumpTree(union treeNode *treeRoot);

void dumpTreeNode(union treeNode *tree_node);

void dumpTreeNodePreorder(union treeNode *tree_node);

void dumpTreeNodeInorder(union treeNode *tree_node);

#endif // _bb188d3c_e320_11dc_8b9d_00502c05c241_

