#ifndef _4f536201_ec79_11dc_b9dd_00502c05c241_
#define _4f536201_ec79_11dc_b9dd_00502c05c241_
#include "include_test_2.h"
/*include test 1 */
__DATE__ __TIME__

#define ARRAY_SIZE 16 //{aaa,bbb}
#undef ARRAY_SIZE //{ccc}
#define ARRAY_SIZE 32 //{ccc}

int array[ARRAY_SIZE]; 

#line 21 "original_file"
#pragma pack
#error Not a conforming C++ compiler.
#endif // _4f536201_ec79_11dc_b9dd_00502c05c241_

