/*
 * UUID: d06a9c9c-f594-11dc-8f19-00502c05c241
 * Author: Brian M. Ames, bames@apk.net
 * Copyright: Copyright (C) 2008 by Brian M. Ames
 */

#ifndef _d06a9c9c_f594_11dc_8f19_00502c05c241_
#define _d06a9c9c_f594_11dc_8f19_00502c05c241_

#ifndef __USE_GNU

char *strndup(const char *S, size_t SIZE);

#endif  // ifndef __USE_GNU

char *strdup_less(const char *str);

#endif  // ifndef _d06a9c9c_f594_11dc_8f19_00502c05c241_
